"use strict";
/*
 * Copyright (c) 2018-2020, salesforce.com, inc.
 * All rights reserved.
 * SPDX-License-Identifier: BSD-3-Clause
 * For full license text, see the LICENSE file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidationUtil = void 0;
class ValidationUtil {
    /**
     * Returns true if the fieldname is a valid api name
     *
     * @param  fieldName API name of the object
     */
    validateAPIName(fieldName) {
        // trimming the __c from the field during character count since it does not count towards the limit
        // Regex makes sure that the field name is alpha numeric, doesn't end in an underscore
        // and optionally if it ends in __c
        return fieldName.replace('__c', '').replace('__C', '').length <= 40 && /^[a-zA-Z][a-zA-Z0-9]*(_[a-zA-Z0-9]+)*(__[cC])?$/.test(fieldName);
    }
    /**
     * Returns true if the fieldname is a valid metadata object name
     *
     * @param  fieldName API name of the field
     */
    validateMetadataTypeName(typeName) {
        // trimming the __mdt from the field during character count since it does not count towards the limit
        // Regex makes sure that the field name is alpha numeric, doesn't end in an underscore
        // and optionally if it ends in __mdt
        return typeName.replace('__mdt', '').length <= 40 && /^[a-zA-Z][a-zA-Z0-9]*(_[a-zA-Z0-9]+)*(__mdt)?$/.test(typeName);
    }
    /**
     * Returns true if the fieldname is a valid metadata record name
     *
     * @param  fieldName record name of a metadata record
     */
    validateMetadataRecordName(typeName) {
        // Regex makes sure that the field name is alpha numeric, doesn't end in an underscore
        return typeName.length <= 40 && /^[a-zA-Z][a-zA-Z0-9]*(_[a-zA-Z0-9]+)*$/.test(typeName);
    }
    /**
     * Returns true if name is below 40 characters
     *
     * @param  name label name or plural label
     */
    validateLessThanForty(name) {
        return name.length <= 40;
    }
}
exports.ValidationUtil = ValidationUtil;
//# sourceMappingURL=validationUtil.js.map